# Generated by Django 4.0.8 on 2022-12-22 01:51

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0078_characterwalletjournalentry_corptools_c_date_aebcea_idx_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='characteraudit',
            name='last_update_location',
            field=models.DateTimeField(blank=True, default=None, null=True),
        ),
        migrations.CreateModel(
            name='CharacterLocation',
            fields=[
                ('id', models.AutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('character', models.OneToOneField(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.characteraudit')),
                ('current_location', models.ForeignKey(blank=True, default=None, null=True,
                 on_delete=django.db.models.deletion.CASCADE, to='corptools.evelocation')),
                ('current_ship', models.ForeignKey(blank=True, default=None, null=True,
                 on_delete=django.db.models.deletion.CASCADE, to='corptools.eveitemtype')),
            ],
        ),
    ]



# Generated by Django 2.2.12 on 2020-05-20 09:18

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0012_skilltotals'),
    ]

    operations = [
        migrations.CreateModel(
            name='EveItemDogmaAttribute',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('attribute_id', models.BigIntegerField(default=None, null=True)),
                ('value', models.FloatField(default=None, null=True)),
                ('eve_type', models.ForeignKey(default=None, null=True,
                                               on_delete=django.db.models.deletion.SET_NULL, to='corptools.EveItemType')),
            ],
        ),
    ]



# Generated by Django 3.1.1 on 2021-01-12 01:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0037_mapsystemmoon_mapsystemplanet'),
    ]

    operations = [
        migrations.CreateModel(
            name='FullyLoadedFilter',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=500)),
                ('description', models.CharField(max_length=500)),
            ],
            options={
                'verbose_name': 'Smart Filter: Audit Fully Loaded',
                'verbose_name_plural': 'Smart Filter: Audit Fully Loaded',
            },
        ),
        migrations.AlterField(
            model_name='characterasset',
            name='blueprint_copy',
            field=models.BooleanField(default=None, null=True),
        ),
        migrations.AlterField(
            model_name='charactermarketorder',
            name='is_buy_order',
            field=models.BooleanField(default=None, null=True),
        ),
        migrations.AlterField(
            model_name='corpasset',
            name='blueprint_copy',
            field=models.BooleanField(default=None, null=True),
        ),
        migrations.AlterField(
            model_name='corporationhistory',
            name='is_deleted',
            field=models.BooleanField(default=None, null=True),
        ),
        migrations.AlterField(
            model_name='corporationmarketorder',
            name='is_buy_order',
            field=models.BooleanField(default=None, null=True),
        ),
        migrations.AlterField(
            model_name='notification',
            name='is_read',
            field=models.BooleanField(default=None, null=True),
        ),
        migrations.CreateModel(
            name='Skillfilter',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=500)),
                ('description', models.CharField(max_length=500)),
                ('required_skill_lists', models.ManyToManyField(
                    to='corptools.SkillList')),
                ('single_req_skill_lists', models.ManyToManyField(
                    blank=True, related_name='single_req', to='corptools.SkillList')),
            ],
            options={
                'verbose_name': 'Smart Filter: Skill list checks',
                'verbose_name_plural': 'Smart Filter: Skill list checks',
            },
        ),
        migrations.CreateModel(
            name='AssetsFilter',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=500)),
                ('description', models.CharField(max_length=500)),
                ('eve_types', models.ManyToManyField(to='corptools.EveItemType')),
                ('systems', models.ManyToManyField(
                    blank=True, to='corptools.MapSystem')),
            ],
            options={
                'verbose_name': 'Smart Filter: Assets and Locations',
                'verbose_name_plural': 'Smart Filter: Assets and Locations',
            },
        ),
    ]



# Generated by Django 2.2.9 on 2020-05-08 15:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0005_auto_20200508_1506'),
    ]

    operations = [
        migrations.AddField(
            model_name='invtypematerials',
            name='type_id',
            field=models.IntegerField(default=0),
            preserve_default=False,
        ),
    ]



# Generated by Django 3.2.8 on 2022-01-09 10:19

from django.db import migrations
from django.db.models import F

from corptools.models import NotificationText


def migrate_notifications(apps, schema_editor):
    Notification = apps.get_model('corptools', 'Notification')
    cnt = Notification.objects.filter(note_text__isnull=True).count()
    print("")
    print(f"Starting Sync {cnt} to sync")
    while cnt > 0:
        n = list(Notification.objects.filter(
            note_text__isnull=True).values_list('pk', flat=True)[:50000])
        nc = Notification.objects.filter(pk__in=n).update(
            note_text_id=F("notification_id"))
        cnt = Notification.objects.filter(note_text__isnull=True).count()
        print(f"synced {nc}. {cnt} remaining")


class Migration(migrations.Migration):
    atomic = False
    dependencies = [
        ('corptools', '0064_notification_note_text'),
    ]

    operations = [
        migrations.RunPython(migrate_notifications)
    ]



from django.conf import settings

CORPTOOLS_APP_NAME = getattr(settings, "CORPTOOLS_APP_NAME", "Character Audit")

CORPTOOLS_DISCORD_BOT_COGS = getattr(
    settings, 'CORPTOOLS_DISCORD_BOT_COGS', ["corptools.cogs.routes", "corptools.cogs.locate", "corptools.cogs.where"])

CT_CHAR_MAX_INACTIVE_DAYS = getattr(settings, 'CT_CHAR_MAX_INACTIVE_DAYS', 3)

CT_CHAR_ASSETS_MODULE = getattr(settings, 'CT_CHAR_ASSETS_MODULE', True)
CT_CHAR_STANDINGS_MODULE = getattr(settings, 'CT_CHAR_STANDINGS_MODULE', True)
CT_CHAR_KILLMAILS_MODULE = getattr(settings, 'CT_CHAR_KILLMAILS_MODULE', True)
CT_CHAR_FITTINGS_MODULE = getattr(settings, 'CT_CHAR_FITTINGS_MODULE', True)
CT_CHAR_CALENDAR_MODULE = getattr(settings, 'CT_CHAR_CALENDAR_MODULE', True)
CT_CHAR_CONTACTS_MODULE = getattr(settings, 'CT_CHAR_CONTACTS_MODULE', True)
CT_CHAR_NOTIFICATIONS_MODULE = getattr(
    settings, 'CT_CHAR_NOTIFICATIONS_MODULE', True)
CT_CHAR_ROLES_MODULE = getattr(settings, 'CT_CHAR_ROLES_MODULE', True)
CT_CHAR_INDUSTRY_MODULE = getattr(settings, 'CT_CHAR_INDUSTRY_MODULE', True)
CT_CHAR_MINING_MODULE = getattr(settings, 'CT_CHAR_MINING_MODULE', True)
CT_CHAR_WALLET_MODULE = getattr(settings, 'CT_CHAR_WALLET_MODULE', True)
CT_CHAR_SKILLS_MODULE = getattr(settings, 'CT_CHAR_SKILLS_MODULE', True)
CT_CHAR_CLONES_MODULE = getattr(settings, 'CT_CHAR_CLONES_MODULE', True)
CT_CHAR_LOCATIONS_MODULE = getattr(settings, 'CT_CHAR_LOCATIONS_MODULE', True)
CT_CHAR_FLEET_MODULE = getattr(settings, 'CT_CHAR_FLEET_MODULE', True)
CT_CHAR_MAIL_MODULE = getattr(settings, 'CT_CHAR_MAIL_MODULE', False)
CT_CHAR_HELPER_MODULE = getattr(settings, 'CT_CHAR_HELPER_MODULE', True)
CT_CHAR_OPPORTUNITIES = False
CT_CHAR_LOYALTYPOINTS_MODULE = getattr(
    settings, 'CT_CHAR_LOYALTYPOINTS_MODULE', True)

CT_CHAR_ACTIVE_IGNORE_CORP_HISTORY = getattr(
    settings, 'CT_CHAR_ACTIVE_IGNORE_CORP_HISTORY', True)
CT_CHAR_ACTIVE_IGNORE_ASSETS_MODULE = getattr(
    settings, 'CT_CHAR_ACTIVE_IGNORE_ASSETS_MODULE', False)
CT_CHAR_ACTIVE_IGNORE_STANDINGS_MODULE = getattr(
    settings, 'CT_CHAR_ACTIVE_IGNORE_STANDINGS_MODULE', False)
CT_CHAR_ACTIVE_IGNORE_KILLMAILS_MODULE = getattr(
    settings, 'CT_CHAR_ACTIVE_IGNORE_KILLMAILS_MODULE', False)
CT_CHAR_ACTIVE_IGNORE_FITTINGS_MODULE = getattr(
    settings, 'CT_CHAR_ACTIVE_IGNORE_FITTINGS_MODULE', False)
CT_CHAR_ACTIVE_IGNORE_CALLENDAR_MODULE = getattr(
    settings, 'CT_CHAR_ACTIVE_IGNORE_CALLENDAR_MODULE', False)
CT_CHAR_ACTIVE_IGNORE_CONTACTS_MODULE = getattr(
    settings, 'CT_CHAR_ACTIVE_IGNORE_CONTACTS_MODULE', False)
CT_CHAR_ACTIVE_IGNORE_NOTIFICATIONS_MODULE = getattr(
    settings, 'CT_CHAR_ACTIVE_IGNORE_NOTIFICATIONS_MODULE', False)
CT_CHAR_ACTIVE_IGNORE_ROLES_MODULE = getattr(
    settings, 'CT_CHAR_ACTIVE_IGNORE_ROLES_MODULE', False)
CT_CHAR_ACTIVE_IGNORE_INDUSTRY_MODULE = getattr(
    settings, 'CT_CHAR_ACTIVE_IGNORE_INDUSTRY_MODULE', False)
CT_CHAR_ACTIVE_IGNORE_WALLET_MODULE = getattr(
    settings, 'CT_CHAR_ACTIVE_IGNORE_WALLET_MODULE', False)
CT_CHAR_ACTIVE_IGNORE_SKILLS_MODULE = getattr(
    settings, 'CT_CHAR_ACTIVE_IGNORE_SKILLS_MODULE', False)
CT_CHAR_ACTIVE_IGNORE_CLONES_MODULE = getattr(
    settings, 'CT_CHAR_ACTIVE_IGNORE_CLONES_MODULE', False)
CT_CHAR_ACTIVE_IGNORE_LOCATIONS_MODULE = getattr(
    settings, 'CT_CHAR_ACTIVE_IGNORE_LOCATIONS_MODULE', False)
CT_CHAR_ACTIVE_IGNORE_MAIL_MODULE = getattr(
    settings, 'CT_CHAR_ACTIVE_IGNORE_MAIL_MODULE', False)
CT_CHAR_ACTIVE_IGNORE_HELPER_MODULE = getattr(
    settings, 'CT_CHAR_ACTIVE_IGNORE_HELPER_MODULE', False)

CT_CHAR_PAUSE_CONTRACTS = getattr(settings, 'CT_CHAR_PAUSE_CONTRACTS', False)

# New scopes so existing all will fail so lets ignore this by default.
CT_CHAR_ACTIVE_IGNORE_LOYALTYPOINTS_MODULE = getattr(
    settings, 'CT_CHAR_ACTIVE_IGNORE_LOYALTYPOINTS_MODULE', True)

# Swap to true for a release or 2
CT_CHAR_ACTIVE_IGNORE_MINING_MODULE = getattr(
    settings, 'CT_CHAR_ACTIVE_IGNORE_MINING_MODULE', True)


def get_character_scopes():
    _scopes = [
        # Base
        'publicData',
    ]

    if CT_CHAR_HELPER_MODULE:
        _scopes += [
            # Helpers
            'esi-ui.open_window.v1',
            'esi-ui.write_waypoint.v1',
        ]

    if CT_CHAR_STANDINGS_MODULE:
        _scopes += [
            # Standings
            'esi-characters.read_standings.v1',
        ]

    if CT_CHAR_KILLMAILS_MODULE:
        _scopes += [
            # Killmails
            'esi-killmails.read_killmails.v1',
        ]

    if CT_CHAR_FITTINGS_MODULE:
        _scopes += [
            # Fittings
            'esi-fittings.read_fittings.v1',
        ]

    if CT_CHAR_CALENDAR_MODULE:
        _scopes += [
            # Callendar
            'esi-calendar.read_calendar_events.v1',
        ]

    if CT_CHAR_CONTACTS_MODULE:
        _scopes += [
            # Contacts
            'esi-characters.read_contacts.v1',
        ]

    if CT_CHAR_NOTIFICATIONS_MODULE:
        _scopes += [
            # Notifications
            'esi-characters.read_notifications.v1',
        ]

    if CT_CHAR_ROLES_MODULE:
        _scopes += [
            # Roles
            'esi-characters.read_titles.v1',
            'esi-characters.read_corporation_roles.v1',
        ]

    if CT_CHAR_INDUSTRY_MODULE:
        _scopes += [
            # Industry
            'esi-industry.read_character_jobs.v1',
        ]

    if CT_CHAR_MINING_MODULE:
        _scopes += [
            # Mining
            'esi-industry.read_character_mining.v1',
        ]

    if CT_CHAR_WALLET_MODULE:
        _scopes += [
            # Wallet / Market /  Contracts
            'esi-markets.read_character_orders.v1',
            'esi-wallet.read_character_wallet.v1',
            'esi-contracts.read_character_contracts.v1',
        ]

    if CT_CHAR_ASSETS_MODULE:
        _scopes += [
            # Assets
            'esi-assets.read_assets.v1',
        ]

    if CT_CHAR_SKILLS_MODULE:
        _scopes += [
            # Skills
            'esi-skills.read_skillqueue.v1',
            'esi-skills.read_skills.v1',
        ]

    if CT_CHAR_CLONES_MODULE:
        _scopes += [
            # Clones
            'esi-clones.read_implants.v1',
            'esi-clones.read_clones.v1',
        ]

    if CT_CHAR_LOCATIONS_MODULE:
        _scopes += [
            # Locations
            'esi-location.read_location.v1',
            'esi-location.read_online.v1',
            'esi-location.read_ship_type.v1',
            'esi-characters.read_fatigue.v1',
        ]

    if CT_CHAR_FLEET_MODULE:
        _scopes += [
            # Fleets
            'esi-fleets.read_fleet.v1',
            'esi-fleets.write_fleet.v1',
        ]

    if CT_CHAR_MAIL_MODULE:
        _scopes += [
            # Mailsn
            'esi-mail.read_mail.v1',
        ]

    # if CT_CHAR_OPPORTUNITIES:
    #     _scopes += [
    #         'esi-characters.read_opportunities.v1',
    #     ]

    if CT_CHAR_CLONES_MODULE or CT_CHAR_ASSETS_MODULE or CT_CHAR_WALLET_MODULE or CT_CHAR_MINING_MODULE:
        _scopes += [
            'esi-universe.read_structures.v1',
            'esi-search.search_structures.v1',
        ]

    if CT_CHAR_LOYALTYPOINTS_MODULE:
        _scopes += [
            'esi-characters.read_loyalty.v1',
        ]

    return list(set(_scopes))


def get_character_update_attributes():
    _attribs = [
        # Base
        ("Public Data", 'last_update_pub_data'),
    ]

    if CT_CHAR_CONTACTS_MODULE:
        _attribs += [
            # Contacts
            ("Contacts", 'last_update_contacts'),
        ]

    if CT_CHAR_LOCATIONS_MODULE:
        _attribs += [
            # Location
            ("Location", 'last_update_location'),
        ]

    if CT_CHAR_NOTIFICATIONS_MODULE:
        _attribs += [
            # Notifications
            ("Notifications", 'last_update_notif'),
        ]
    if CT_CHAR_NOTIFICATIONS_MODULE:
        _attribs += [
            # Notifications
            ("Mining", 'last_update_mining'),
        ]

    if CT_CHAR_ROLES_MODULE:
        _attribs += [
            # Roles
            ("Roles", 'last_update_roles'),
            ("Titles", 'last_update_titles'),
        ]

    if CT_CHAR_WALLET_MODULE:
        _attribs += [
            # Wallet / Market /  Contracts
            ("Wallet", 'last_update_wallet'),
            ("Orders", 'last_update_orders'),
        ]

    if CT_CHAR_INDUSTRY_MODULE:
        _attribs += [
            # Roles
            ("Industry", 'last_update_indy'),
        ]

    if CT_CHAR_LOYALTYPOINTS_MODULE:
        _attribs += [
            ("LP", 'last_update_loyaltypoints'),
        ]

    if CT_CHAR_ASSETS_MODULE:
        _attribs += [
            # Assets
            ("Assets", 'last_update_assets'),
        ]

    if CT_CHAR_SKILLS_MODULE:
        _attribs += [
            # Skills
            ("Skills", 'last_update_skills'),
            ("Skill Queue", 'last_update_skill_que')
        ]

    if CT_CHAR_CLONES_MODULE:
        _attribs += [
            # Clones
            ("Clones", 'last_update_clones'),
        ]

    if CT_CHAR_MAIL_MODULE:
        _attribs += [
            # Mail
            ("Mail", 'last_update_mails'),
        ]

    return _attribs


def get_corp_update_attributes():
    _attribs = [
        # Base
        ["Assets", 'last_update_assets', "a"],
        ("Structures", 'last_update_structures', "s"),
        ("Wallet", 'last_update_wallet', "w"),
        ("Moons Observations", 'last_update_observers', "m")
    ]

    return _attribs


_corp_scopes_base = [
    # All...
    'esi-search.search_structures.v1',
    'esi-universe.read_structures.v1',
    'esi-characters.read_corporation_roles.v1',
]

# Tracking
_corp_scopes_tracking = [
    'esi-corporations.track_members.v1',
    'esi-corporations.read_titles.v1',
    'esi-corporations.read_corporation_membership.v1',
    'esi-killmails.read_corporation_killmails.v1',
]

# structures
_corp_scopes_structures = [
    'esi-planets.read_customs_offices.v1',
    'esi-corporations.read_structures.v1',
]

# structures
_corp_scopes_starbases = [
    'esi-corporations.read_starbases.v1',
]

# moons
_corp_scopes_moons = [
    'esi-industry.read_corporation_mining.v1',
]

# wallets
_corp_scopes_wallets = [
    'esi-wallet.read_corporation_wallets.v1',
    'esi-markets.read_corporation_orders.v1',
    'esi-industry.read_corporation_jobs.v1',
    'esi-corporations.read_divisions.v1',
]

# assets
_corp_scopes_assets = [
    'esi-assets.read_corporation_assets.v1',
]

_corp_scopes_contracts = [
    "esi-contracts.read_corporation_contracts.v1",
]

_corp_scopes_pocos = [
    'esi-planets.read_customs_offices.v1',
]


CORP_REQUIRED_SCOPES = _corp_scopes_base + _corp_scopes_tracking + \
    _corp_scopes_structures + _corp_scopes_moons + \
    _corp_scopes_wallets + _corp_scopes_assets

CT_BETA_UI = getattr(
    settings, 'CT_BETA_UI', False)

CT_PAGINATION_SIZE = getattr(
    settings, 'CT_PAGINATION_SIZE', 30000)

CT_USERS_CAN_FORCE_REFRESH = getattr(
    settings, 'CT_USERS_CAN_FORCE_REFRESH', False)

CT_TASK_SPREAD_DELAY = getattr(
    settings, 'CT_TASK_SPREAD_DELAY', 1200)  # Default 20 minute spread



LOCATION_FLAG_REPLACEMENTS = {
    "unknown location_flag (185)": "InfrastructureHangar",
    "unknown location_flag (186)": "MoonMaterialBay"
}

NOTIFICATION_TYPE_REPLACEMENTS = {
    "unknown notification type (263)": "CorporationGoalNameChange",
    "unknown notification type (264)": "CorporationGoalExpired",
    "unknown notification type (265)": "CorporationGoalLimitReached",
    "unknown notification type (276)": "SPAutoRedeemed",
    "unknown notification type (280)": "SkinSequencingCompleted",
    "unknown notification type (281)": "SkyhookOnline",
    "unknown notification type (282)": "SkyhookLostShields",
    "unknown notification type (283)": "SkyhookUnderAttack",
    "unknown notification type (284)": "SkyhookDestroyed",
    "unknown notification type (285)": "SkyhookDeployed",
    "unknown notification type (286)": "MercenaryDenReinforced",
    "unknown notification type (287)": "MercenaryDenAttacked",
    "unknown notification type (6013)": "DailyItemRewardAutoClaimed",
    "unknown notification type (6040)": "StructureLowReagentsAlert",
    "unknown notification type (6041)": "StructureNoReagentsAlert"
}


def sanitize_location_flag(flag):
    return LOCATION_FLAG_REPLACEMENTS.get(flag, flag)


def sanitize_notification_type(note_type):
    return NOTIFICATION_TYPE_REPLACEMENTS.get(note_type, note_type)



# Generated by Django 2.2.9 on 2020-05-10 13:57

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0006_invtypematerials_type_id'),
    ]

    operations = [
        migrations.AlterField(
            model_name='skillqueue',
            name='skill_name',
            field=models.ForeignKey(
                default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='corptools.EveItemType'),
        ),
    ]



# Generated by Django 2.2.12 on 2020-08-04 09:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0020_clone'),
    ]

    operations = [
        migrations.AlterField(
            model_name='jumpclone',
            name='location_id',
            field=models.BigIntegerField(default=None, null=True),
        ),
        migrations.AlterField(
            model_name='jumpclone',
            name='location_type',
            field=models.CharField(choices=[(
                'station', 'station'), ('structure', 'structure')], default=None, max_length=9, null=True),
        ),
    ]



import logging
from typing import List

from ninja import NinjaAPI

from django.db.models import F, Q, Sum
from django.db.models.functions import Length

from allianceauth.services.hooks import get_extension_logger

from corptools import models
from corptools.api import schema

logger = get_extension_logger(__name__)


class AssetsApiEndpoints:

    tags = ["Assets"]

    def __init__(self, api: NinjaAPI):
        @api.get(
            "corporation/{corporation_id}/asset/locations",
            response={200: List[schema.ValueLabel], 403: schema.Message},
            tags=self.tags
        )
        def get_corporation_asset_locations(request, corporation_id: int, top_level_only: bool = True):
            perms = (
                request.user.has_perm('corptools.own_corp_manager')
                | request.user.has_perm('corptools.alliance_corp_manager')
                | request.user.has_perm('corptools.state_corp_manager')
                | request.user.has_perm('corptools.global_corp_manager')
                | request.user.has_perm('corptools.holding_corp_assets')
            )

            if not perms:
                logging.error(
                    f"Permission Denied for {request.user} to view wallets!")
                return 403, "Permission Denied!"

            asset_locs = models.CorpAsset.get_visible(request.user).filter(
                corporation__corporation__corporation_id=corporation_id,
                location_name__isnull=False).values_list('location_name').distinct()
            asset_locs = models.EveLocation.objects.filter(
                location_id__in=asset_locs,
            ).order_by('location_name')
            if top_level_only:
                asset_locs = asset_locs.filter(managed=False)
            asset_locations = [{"label": "Everywhere", "value": 0},
                               {"label": "AssetSafety", "value": 2004}, ]
            for loc in asset_locs:
                asset_locations.append({
                    "label": loc.location_name,
                    "value": loc.location_id
                })

            return asset_locations

        @api.get(
            "corporation/{corporation_id}/asset/{location_id}/list",
            response={200: List[schema.CorporationAssetItem],
                      403: schema.Message},
            tags=self.tags
        )
        def get_corporation_asset_list(request, corporation_id: int, location_id: int, new_asset_tree: bool = False):
            perms = (
                request.user.has_perm('corptools.own_corp_manager')
                | request.user.has_perm('corptools.alliance_corp_manager')
                | request.user.has_perm('corptools.state_corp_manager')
                | request.user.has_perm('corptools.global_corp_manager')
                | request.user.has_perm('corptools.holding_corp_assets')
            )

            if not perms:
                logging.error(
                    f"Permission Denied for {request.user} to view wallets!")
                return 403, "Permission Denied!"

            if new_asset_tree:
                expandable_cats = []

                if corporation_id == 0:
                    corporation_id = request.user.profile.main_character.corporation_id

                assets = models.CorpAsset.get_visible(request.user).filter(
                    corporation__corporation__corporation_id=corporation_id).select_related(
                    "type_name", "location_name", "type_name__group__category"
                )

                asset_locations = []
                if location_id == 2004:
                    asset_locations = assets.filter(
                        location_flag="AssetSafety")
                    assets = assets.filter(
                        location_id__in=asset_locations.values_list('item_id'))
                elif location_id != 0:
                    asset_locations = assets.filter(
                        location_name_id=int(location_id))
                    assets = assets.filter(Q(location_name_id=int(location_id)) | Q(
                        location_id__in=asset_locations.values_list('item_id')) | Q(location_id=int(location_id)))
                else:
                    asset_locations = assets.filter(
                        location_name__isnull=False)

                output = []
                location_names = {}

                for a in assets:
                    loc = f"{a.location_id} ({a.location_flag})"
                    if a.location_name:
                        loc = a.location_name.location_name
                    output.append({
                        "item": {
                            "id": a.type_name.type_id,
                            "name": a.type_name.name,
                            "cat": f"{a.type_name.group.category.name} - {a.type_name.group.name}"
                        },
                        "quantity": a.quantity,
                        "id": a.item_id,
                        "expand": False,
                        "location": {
                            "id": a.location_id,
                            "name": loc
                        },

                    })

                return output

            expandable_cats = [2, 6, 29]
            everywhere_flags = ["CorpSAG1", "CorpSAG2", "CorpSAG3", "CorpSAG4",
                                "CorpSAG5", "CorpSAG6", "CorpSAG7", "CorpDeliveries", "AssetSafety"]

            if corporation_id == 0:
                corporation_id = request.user.profile.main_character.corporation_id

            assets = models.CorpAsset.get_visible(request.user).filter(
                corporation__corporation__corporation_id=corporation_id).select_related(
                "type_name", "location_name", "type_name__group__category"
            )

            asset_locations = []
            if location_id == 2004:
                asset_locations = assets.filter(
                    location_flag="AssetSafety")
                assets = assets.filter(
                    location_id__in=asset_locations.values_list('item_id'))
            elif location_id != 0:
                asset_locations = assets.filter(
                    location_name_id=int(location_id))
                assets = assets.filter(Q(location_name_id=int(location_id)) | Q(
                    location_id__in=asset_locations.values_list('item_id')) | Q(location_id=int(location_id)))
            else:
                asset_locations = assets.filter(
                    location_name__isnull=False)
                assets = assets.filter(location_flag__in=everywhere_flags)

            output = []
            location_names = {}
            for lo in asset_locations:
                if lo.location_name:
                    location_names[lo.item_id] = lo.location_name.location_name

            for a in assets:
                loc = a.location_flag
                if a.location_id in location_names:
                    loc = f"{location_names[a.location_id]} ({a.location_flag})"

                output.append({
                    "item": {
                        "id": a.type_name.type_id,
                        "name": a.type_name.name,
                        "cat": f"{a.type_name.group.category.name} - {a.type_name.group.name}"
                    },
                    "quantity": a.quantity,
                    "id": a.item_id,
                    "expand": True if a.type_name.group.category.category_id in expandable_cats else False,
                    "location": {
                        "id": a.location_id,
                        "name": loc
                    }
                })

            return output

        @api.get(
            "corporation/asset/{item_id}/contents",
            response={200: List[schema.CorporationAssetItem],
                      403: schema.Message},
            tags=self.tags
        )
        def get_corporation_asset_contents(request, item_id: int):
            perms = (
                request.user.has_perm('corptools.own_corp_manager')
                | request.user.has_perm('corptools.alliance_corp_manager')
                | request.user.has_perm('corptools.state_corp_manager')
                | request.user.has_perm('corptools.global_corp_manager')
                | request.user.has_perm('corptools.holding_corp_assets')
            )

            if not perms:
                logging.error(
                    f"Permission Denied for {request.user} to view wallets!")
                return 403, "Permission Denied!"

            assets = models.CorpAsset.get_visible(request.user)\
                .select_related(
                    "type_name", "location_name", "type_name__group__category"
            )
            assets = assets.filter(location_id=item_id)

            output = []

            for a in assets:
                output.append({
                    "item": {
                        "id": a.type_name.type_id,
                        "name": a.type_name.name,
                        "cat": f"{a.type_name.group.category.name} - {a.type_name.group.name}"
                    },
                    "quantity": a.quantity,
                    "id": a.item_id,
                    "expand": False,
                    "location": {
                        "id": item_id,
                        "name": a.location_flag
                    }
                })

            return output

        @api.get(
            "corporation/{corporation_id}/asset/{location_id}/groups",
            response={200: List[schema.CharacterAssetGroups],
                      403: schema.Message},
            tags=self.tags
        )
        def get_corporation_asset_groups(request, corporation_id: int, location_id: int):
            perms = (
                request.user.has_perm('corptools.own_corp_manager')
                | request.user.has_perm('corptools.alliance_corp_manager')
                | request.user.has_perm('corptools.state_corp_manager')
                | request.user.has_perm('corptools.global_corp_manager')
                | request.user.has_perm('corptools.holding_corp_assets')
            )

            if not perms:
                logging.error(
                    f"Permission Denied for {request.user} to view wallets!")
                return 403, "Permission Denied!"

            capital_groups = [30, 547, 659, 1538, 485, 902, 513, 883, 77283]
            subcap_cat = [6]
            noteable_cats = [4, 20, 23, 25, 34, 35, 87, 91]
            structure_cats = [22, 24, 40, 41, 46, 65, 66, ]
            bpo_cats = [9]

            assets = models.CorpAsset.get_visible(request.user)\
                .filter((Q(blueprint_copy=None) | Q(blueprint_copy=False)),
                        corporation__corporation__corporation_id=corporation_id)

            if location_id == 2004:
                asset_locations = assets.filter(
                    location_flag="AssetSafety").values_list('item_id')
                assets = assets.filter(location_id__in=asset_locations)
            elif location_id != 0:
                asset_locations = assets.filter(
                    location_name_id=int(location_id)).values_list('item_id')
                assets = assets.filter(Q(location_name_id=int(location_id)) | Q(
                    location_id__in=asset_locations) | Q(location_id=int(location_id)))

            assets = assets.values('type_name__group__group_id')\
                .annotate(grp_total=Sum('quantity'))\
                .annotate(grp_name=F('type_name__group__name'))\
                .annotate(grp_id=F('type_name__group_id'))\
                .annotate(cat_id=F('type_name__group__category_id'))\
                .order_by('-grp_total')

            capital_asset_groups = []
            subcap_asset_groups = []
            noteable_asset_groups = []
            structure_asset_groups = []
            bpo_asset_groups = []
            remaining_asset_groups = []

            for grp in assets:
                _grp = {
                    "label": grp['grp_name'],
                    "value": grp['grp_total'],
                }
                if grp['grp_id'] in capital_groups:
                    capital_asset_groups.append(_grp)
                elif grp['cat_id'] in subcap_cat:
                    subcap_asset_groups.append(_grp)
                elif grp['cat_id'] in noteable_cats:
                    noteable_asset_groups.append(_grp)
                elif grp['cat_id'] in structure_cats:
                    structure_asset_groups.append(_grp)
                elif grp['cat_id'] in bpo_cats:
                    bpo_asset_groups.append(_grp)
                else:
                    remaining_asset_groups.append(_grp)

            return [
                {"name": "Capital Ships",
                 "items": capital_asset_groups},
                {"name": "Subcap Ships",
                 "items": subcap_asset_groups},
                {"name": "Noteable Assets",
                 "items": noteable_asset_groups},
                {"name": "Structures",
                 "items": structure_asset_groups},
                {"name": "BPO",
                 "items": bpo_asset_groups},
                {"name": "Remaining",
                 "items": remaining_asset_groups},
            ]



from ninja import NinjaAPI

from django.db.models import F, Q, Sum
from django.utils.translation import gettext as _

from allianceauth.services.hooks import get_extension_logger

from corptools import models
from corptools.api.helpers import (
    get_alts_queryset, get_main_character, glance_incursion_check,
    glances_assets_character, glances_gas_check, glances_ice_check,
    glances_industry_check, glances_market_check, glances_missions_check,
    glances_moon_check, glances_ore_check, glances_pi_check,
    glances_pochven_check, glances_ratting_check, roundFloat,
)

# from corptools.api import schema


logger = get_extension_logger(__name__)


class GlanceApiEndpoints:

    tags = ["Character At a Glance"]

    def __init__(self, api: NinjaAPI):
        @api.get(
            "account/{character_id}/glance/assets",
            response={200: dict, 403: str},
            tags=self.tags
        )
        def get_glance_assets(request, character_id: int):
            if character_id == 0:
                character_id = request.user.profile.main_character.character_id

            response, main = get_main_character(request, character_id)

            if not response:
                return 403, _("Permission Denied")

            characters = get_alts_queryset(main)

            return glances_assets_character(characters)

        @api.get(
            "account/{character_id}/glance/activities",
            response={200: dict, 403: str},
            tags=self.tags
        )
        def get_glance_activities(request, character_id: int):
            if character_id == 0:
                character_id = request.user.profile.main_character.character_id

            response, main = get_main_character(request, character_id)

            if not response:
                return 403, _("Permission Denied")

            characters = get_alts_queryset(main)

            output = {}

            output["ratting"] = roundFloat(glances_ratting_check(characters))
            output["incursion"] = roundFloat(
                glance_incursion_check(characters))
            output["pochven"] = roundFloat(glances_pochven_check(characters))
            output["mission"] = roundFloat(glances_missions_check(characters))

            output["market"] = roundFloat(glances_market_check(characters))
            output["industry"] = glances_industry_check(characters)

            output["pi"] = roundFloat(glances_pi_check(characters))

            output["mining_ore"] = roundFloat(glances_ore_check(characters))
            output["mining_moon"] = roundFloat(glances_moon_check(characters))
            output["mining_gas"] = roundFloat(glances_gas_check(characters))
            output["mining_ice"] = roundFloat(glances_ice_check(characters))

            return output

        @api.get(
            "account/{character_id}/glance/faction",
            response={200: dict, 403: str},
            tags=self.tags
        )
        def get_glance_factions(request, character_id: int):
            if character_id == 0:
                character_id = request.user.profile.main_character.character_id

            response, main = get_main_character(request, character_id)

            if not response:
                return 403, _("Permission Denied")

            characters = get_alts_queryset(main)

            output = {
                "factions": {
                    "amarr": 0,
                    "caldari": 0,
                    "gallente": 0,
                    "minmatar": 0,
                    "angel": 0,
                    "guristas": 0,
                },
                "lp": {
                    "total": 0,
                    "evermark": [],
                    "top_five": []
                }
            }

            # Check for any militia stuff
            for character in characters:
                if character.faction_id is not None:
                    # check_faction
                    if character.faction_id == 500003:
                        output["factions"]["amarr"] += 1
                    elif character.faction_id == 500010:
                        output["factions"]["guristas"] += 1
                    elif character.faction_id == 500004:
                        output["factions"]["gallente"] += 1
                    elif character.faction_id == 500011:
                        output["factions"]["angel"] += 1
                    elif character.faction_id == 500001:
                        output["factions"]["caldari"] += 1
                    elif character.faction_id == 500002:
                        output["factions"]["minmatar"] += 1

            account_lp = models.LoyaltyPoint.objects.filter(
                character__character__in=characters,
                amount__gte=1
            ).values(
                corp_id=F("corporation__eve_id")
            ).annotate(
                total_lp=Sum("amount")
            ).annotate(
                corp_name=F("corporation__name")
            ).order_by("-total_lp")[:5]

            for lp in account_lp:
                # Evermarks and anything future
                if lp.get('corp_id') in [1000419,]:
                    output["lp"]["evermark"].append({
                        "corp_id": lp.get("corp_id"),
                        "lp": lp.get("total_lp"),
                        "corp_name": lp.get("corp_name"),
                    })
                else:
                    output["lp"]["top_five"].append({
                        "corp_id": lp.get("corp_id"),
                        "lp": lp.get("total_lp"),
                        "corp_name": lp.get("corp_name"),
                    })

            output["lp"]["total"] = models.LoyaltyPoint.objects.filter(
                character__character__in=characters,
            ).exclude(
                corporation__eve_id__in=[1000419,]
            ).aggregate(total_lp=Sum("amount"))["total_lp"]

            return output



# Generated by Django 2.2.12 on 2020-08-14 10:36

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0027_skilllistfile'),
    ]

    operations = [
        migrations.DeleteModel(
            name='SkillListFile',
        ),
    ]



from datetime import datetime

from ninja import NinjaAPI

from django.db.models import F, Q, Sum
from django.utils import timezone
from django.utils.translation import gettext as _

from allianceauth.services.hooks import get_extension_logger

from corptools import models
# from corptools.api import schema
from corptools.api.helpers import (
    get_corporation_characters, glance_incursion_check,
    glances_assets_corporation, glances_gas_check, glances_ice_check,
    glances_industry_check, glances_market_check, glances_missions_check,
    glances_moon_check, glances_ore_check, glances_pi_check,
    glances_pochven_check, glances_ratting_check, roundFloat,
)

logger = get_extension_logger(__name__)


def check_permisions(corp_id, user):
    corps_vis = models.CorporationAudit.objects.visible_to(user)
    if user.has_perm("corptools.holding_corp_structures"):
        corps_holding = models.CorptoolsConfiguration.get_solo().holding_corp_qs()
        corps_vis = corps_vis | corps_holding
    return corps_vis.filter(corporation__corporation_id=corp_id).exists()


class CorpGlanceApiEndpoints:

    tags = ["Corporation At a Glance"]

    def __init__(self, api: NinjaAPI):
        @api.get(
            "corporation/{corporation_id}/glance/assets",
            response={200: dict, 403: str},
            tags=self.tags
        )
        def get_glance_assets_corp(request, corporation_id: int):
            if corporation_id == 0:
                corporation_id = request.user.profile.main_character.corporation_id

            characters = get_corporation_characters(request, corporation_id)

            if not check_permisions(corporation_id, request.user):
                return 403, _("Permission Denied")

            return glances_assets_corporation(characters, corporation_id)

        @api.get(
            "corporation/{corporation_id}/glance/activities",
            response={200: dict, 403: str},
            tags=self.tags
        )
        def get_glance_activities(request, corporation_id: int):
            if corporation_id == 0:
                corporation_id = request.user.profile.main_character.corporation_id

            characters = get_corporation_characters(request, corporation_id)

            if not check_permisions(corporation_id, request.user):
                return 403, _("Permission Denied")

            output = {}

            output["ratting"] = roundFloat(glances_ratting_check(characters))
            output["incursion"] = roundFloat(
                glance_incursion_check(characters))
            output["pochven"] = roundFloat(glances_pochven_check(characters))
            output["mission"] = roundFloat(glances_missions_check(characters))

            output["market"] = glances_market_check(characters)
            output["industry"] = glances_industry_check(characters)

            output["pi"] = glances_pi_check(characters)

            output["mining_ore"] = roundFloat(glances_ore_check(characters))
            output["mining_moon"] = roundFloat(glances_moon_check(characters))
            output["mining_gas"] = roundFloat(glances_gas_check(characters))
            output["mining_ice"] = roundFloat(glances_ice_check(characters))

            return output

        @api.get(
            "corporation/{corporation_id}/glance/faction",
            response={200: dict, 403: str},
            tags=self.tags
        )
        def get_glance_factions(request, corporation_id: int):
            if corporation_id == 0:
                corporation_id = request.user.profile.main_character.corporation_id

            characters = get_corporation_characters(request, corporation_id)

            if not check_permisions(corporation_id, request.user):
                return 403, _("Permission Denied")

            output = {
                "factions": {
                    "amarr": 0,
                    "caldari": 0,
                    "gallente": 0,
                    "minmatar": 0,
                    "angel": 0,
                    "guristas": 0,
                },
                "lp": {
                    "total": 0,
                    "evermark": [],
                    "top_five": []
                }
            }

            # Check for any militia stuff
            for character in characters:
                if character.faction_id is not None:
                    # check_faction
                    if character.faction_id == 500003:
                        output["factions"]["amarr"] += 1
                    elif character.faction_id == 500010:
                        output["factions"]["guristas"] += 1
                    elif character.faction_id == 500004:
                        output["factions"]["gallente"] += 1
                    elif character.faction_id == 500011:
                        output["factions"]["angel"] += 1
                    elif character.faction_id == 500001:
                        output["factions"]["caldari"] += 1
                    elif character.faction_id == 500002:
                        output["factions"]["minmatar"] += 1

            account_lp = models.LoyaltyPoint.objects.filter(
                character__character__in=characters,
                amount__gte=0
            ).values(
                corp_id=F("corporation__eve_id")
            ).annotate(
                total_lp=Sum("amount")
            ).annotate(
                corp_name=F("corporation__name")
            ).order_by("-total_lp")[:5]

            for lp in account_lp:
                # Evermarks and anything future
                if lp.get('corp_id') in [1000419,]:
                    output["lp"]["evermark"].append({
                        "corp_id": lp.get("corp_id"),
                        "lp": lp.get("total_lp"),
                        "corp_name": lp.get("corp_name"),
                    })
                else:
                    output["lp"]["top_five"].append({
                        "corp_id": lp.get("corp_id"),
                        "lp": lp.get("total_lp"),
                        "corp_name": lp.get("corp_name"),
                    })

            output["lp"]["total"] = models.LoyaltyPoint.objects.filter(
                character__character__in=characters,
            ).aggregate(total_lp=Sum("amount"))["total_lp"]

            return output



# Generated by Django 4.0.2 on 2022-04-12 08:34

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0071_alter_corptoolsconfiguration_options'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='corporationaudit',
            options={'permissions': (('own_corp_manager', "Can access own corporations's data."), ('alliance_corp_manager', "Can access other corporations's data for own alliance."), (
                'state_corp_manager', "Can access other corporations's data for own state."), ('global_corp_manager', "Can access all corporations's data."))},
        ),
    ]



# Generated by Django 4.2.5 on 2023-10-02 10:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0093_alter_characteraudit_options'),
    ]

    operations = [
        migrations.CreateModel(
            name='LastLoginfilter',
            fields=[
                ('id', models.AutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=500)),
                ('description', models.CharField(max_length=500)),
                ('days_since_login', models.IntegerField(
                    default=30, help_text="Days since last login of any Character in the accounts Main's Corporation to pass filter.")),
                ('no_data_pass', models.BooleanField(blank=True, default=False,
                 help_text='If there is no data (No Valid Corp Token) for a characters account then should this filter automatically pass.')),
            ],
            options={
                'verbose_name': 'Smart Filter: Last Login Check',
                'verbose_name_plural': 'Smart Filter: Last Login Check',
            },
        ),
        migrations.AddField(
            model_name='characteraudit',
            name='last_known_login',
            field=models.DateTimeField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name='characteraudit',
            name='last_known_logoff',
            field=models.DateTimeField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name='characteraudit',
            name='last_update_login',
            field=models.DateTimeField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name='corporationaudit',
            name='last_update_known_login',
            field=models.DateTimeField(blank=True, default=None, null=True),
        ),
    ]



# Generated by Django 3.1.1 on 2021-01-12 04:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0038_auto_20210112_0102'),
    ]

    operations = [
        migrations.AddField(
            model_name='assetsfilter',
            name='eve_groups',
            field=models.ManyToManyField(
                blank=True, to='corptools.EveItemGroup'),
        ),
        migrations.AlterField(
            model_name='assetsfilter',
            name='eve_types',
            field=models.ManyToManyField(
                blank=True, to='corptools.EveItemType'),
        ),
    ]



from typing import List

from ninja import NinjaAPI

from django.db.models import F

from corptools import models
from corptools.api import schema

# from ninja.decorators import decorate_view

# from django.views.decorators.cache import cache_page

# from corptools.api.decorators import cache_page_data


class ListApiEndpoints:

    tags = ["Account"]

    def __init__(self, api: NinjaAPI):

        @api.get(
            "account/list",
            response={200: List[schema.AccountStatus], 403: str},
            tags=self.tags
        )
        # @decorate_view(cache_page(300))
        def get_account_list(request, orphans=False):
            characters = models.CharacterAudit.objects.visible_eve_characters(
                request.user
            ).filter(
                character_id=F(
                    "character_ownership__user__profile__main_character__character_id")
            ).select_related(
                'character_ownership',
                'character_ownership__user__profile',
                'character_ownership__user__profile__main_character',
                'characteraudit'
            ).prefetch_related(
                'character_ownership__user__character_ownerships',
                'character_ownership__user__character_ownerships__character',
                'character_ownership__user__character_ownerships__character__characteraudit'
            )

            character_ids = []
            output = {}
            for char in characters:
                main = char.character_ownership.user.profile.main_character
                for c in char.character_ownership.user.character_ownerships.all():
                    if char.character_id not in output:
                        output[main.character_id] = {
                            "main": main,
                            "characters": []
                        }
                    active = False
                    try:
                        # active = c.character.characteraudit.is_active()
                        active = c.character.characteraudit.active
                    except models.CharacterAudit.DoesNotExist:
                        pass
                    character_ids.append(c.character.character_id)
                    output[main.character_id]["characters"].append(
                        {
                            "character": c.character,
                            "active": active
                        }
                    )

            if orphans:
                orphans = models.CharacterAudit.objects.visible_to(
                    request.user).exclude(character__character_id__in=character_ids)\
                    .select_related('character__character_ownership',
                                    'character__character_ownership__user__profile',
                                    'character__character_ownership__user__profile__main_character',
                                    'character__characteraudit')\
                    .prefetch_related('character__character_ownership__user__character_ownerships')\
                    .prefetch_related('character__character_ownership__user__character_ownerships__character')\
                    .prefetch_related('character__character_ownership__user__character_ownerships__character__characteraudit')\

                for char in orphans:
                    try:
                        main = char.character.character_ownership.user.profile.main_character
                        if main.character_id not in output:
                            output[main.character_id] = {
                                "main": main,
                                "characters": [{
                                    "character": main,
                                    "active": False
                                }]
                            }
                        active = False
                        try:
                            # active = char.character.characteraudit.is_active()
                            active = char.character.characteraudit.active
                        except models.CharacterAudit.DoesNotExist:
                            pass
                        output[main.character_id]["characters"].append(
                            {
                                "character": char.character,
                                "active": active
                            }
                        )
                    except Exception:
                        main = char.character
                        active = False
                        try:
                            active = char.character.characteraudit.is_active()
                        except models.CharacterAudit.DoesNotExist:
                            pass
                        output[main.character_id] = {
                            "main": main,
                            "orphan": True,
                            "characters": [{
                                "character": char.character,
                                "active": active
                            }]
                        }

            return list(output.values())



from typing import List

from ninja import NinjaAPI
from ninja.pagination import paginate

from django.db.models import Count, F, Sum
from django.utils.translation import gettext_noop as _

from corptools import models
from corptools.api import schema
from corptools.api.helpers import (
    Paginator, get_alts_queryset, get_main_character,
)


class FinancesApiEndpoints:

    tags = ["Finances"]

    def __init__(self, api: NinjaAPI):

        @api.get(
            "account/{character_id}/wallet",
            response={200: List[schema.CharacterWalletEvent], 403: str},
            tags=self.tags
        )
        @paginate(Paginator)
        def get_character_wallet(request, character_id: int, **kwargs):
            if character_id == 0:
                character_id = request.user.profile.main_character.character_id
            response, main = get_main_character(request, character_id)

            if not response:
                return 403, _("Permission Denied")

            characters = get_alts_queryset(main)
            character_list = list(
                characters.values_list("character_id", flat=True))

            wallet_journal = models.CharacterWalletJournalEntry.objects\
                .filter(character__character__in=characters)\
                .select_related('first_party_name', 'second_party_name', 'character__character').order_by('-date')[:35000]

            output = []
            for w in wallet_journal:
                own_account = False
                if (w.second_party_id in character_list and w.first_party_id in character_list):
                    own_account = True

                output.append(
                    {
                        "character": w.character.character,
                        "id": w.entry_id,
                        "date": w.date,
                        "first_party": {
                            "id": w.first_party_id,
                            "name": w.first_party_name.name,
                            "cat": w.first_party_name.category,
                        },
                        "second_party": {
                            "id": w.second_party_id,
                            "name": w.second_party_name.name,
                            "cat": w.second_party_name.category,
                        },
                        "ref_type": w.ref_type,
                        "amount": w.amount,
                        "balance": w.balance,
                        "reason": w.reason,
                        "own_account": own_account
                    })

            return output

        @api.get(
            "account/{character_id}/orders",
            response={200: List[schema.CharacterOrder], 403: str},
            tags=self.tags
        )
        def get_character_orders(request, character_id: int):
            if character_id == 0:
                character_id = request.user.profile.main_character.character_id
            response, main = get_main_character(request, character_id)

            if not response:
                return 403, _("Permission Denied")

            characters = get_alts_queryset(main)

            orders = models.CharacterMarketOrder.objects\
                .filter(character__character__in=characters)\
                .select_related('type_name', 'location_name', 'character__character')

            output = []
            for w in orders:
                output.append(
                    {
                        "character": w.character.character,
                        "date": w.issued,
                        "duration": w.duration,
                        "volume_min": w.min_volume,
                        "volume_remain": w.volume_remain,
                        "volume_total": w.volume_total,
                        "item": {
                            "id": w.type_id,
                            "name": w.type_name.name
                        },
                        "price": w.price,
                        "escrow": w.escrow,
                        "buy_order": w.is_buy_order,
                        "location": {
                            "id": w.location_id,
                            "name": w.location_name.location_name if w.location_name else w.location_id
                        }
                    }
                )

            return output

        @api.get(
            "account/{character_id}/market",
            response={200: schema.CharacterMarket, 403: str},
            tags=self.tags
        )
        def get_character_market(request, character_id: int):
            if character_id == 0:
                character_id = request.user.profile.main_character.character_id
            response, main = get_main_character(request, character_id)

            if not response:
                return 403, _("Permission Denied")

            characters = get_alts_queryset(main)

            market_data_current = models.CharacterMarketOrder.objects\
                .filter(character__character__in=characters, state="active")\
                .select_related('character__character', 'type_name', 'location_name')

            market_data_old = models.CharacterMarketOrder.objects\
                .filter(character__character__in=characters, duration__gt=0)\
                .select_related('character__character', 'type_name', 'location_name')\
                .exclude(state="active")

            output = {"active": [], "expired": [],
                      "total_active": 0, "total_expired": 0}
            for w in market_data_current:
                output['total_active'] += w.price * w.volume_remain
                o = {
                    "character": w.character.character,
                    "date": w.issued,
                    "duration": w.duration,
                    "volume_min": w.min_volume,
                    "volume_remain": w.volume_remain,
                    "volume_total": w.volume_total,
                    "item": {
                        "id": w.type_name.type_id,
                        "name": w.type_name.name
                    },
                    "price": w.price,
                    "escrow": w.escrow,
                    "buy_order": True if w.is_buy_order else False,
                }
                if w.location_name:
                    o['location'] = {
                        "id": w.location_name.location_id,
                        "name": w.location_name.location_name
                    }
                output["active"].append(o)

            for w in market_data_old:
                output['total_expired'] += w.price * w.volume_total
                o = {
                    "character": w.character.character,
                    "date": w.issued,
                    "duration": w.duration,
                    "volume_min": w.min_volume,
                    "volume_remain": w.volume_remain,
                    "volume_total": w.volume_total,
                    "item": {
                        "id": w.type_name.type_id,
                        "name": w.type_name.name
                    },
                    "price": w.price,
                    "escrow": w.escrow,
                    "buy_order": True if w.is_buy_order else False,
                }
                if w.location_name:
                    o['location'] = {
                        "id": w.location_name.location_id,
                        "name": w.location_name.location_name
                    }
                output["expired"].append(o)

            return output

        @api.get(
            "account/{character_id}/wallet/activity",
            tags=self.tags
        )
        def get_character_wallet_activity(request, character_id: int):
            if not (request.user.has_perm("corptools.global_corp_manager")
                    or request.user.has_perm("corptools.state_corp_manager")
                    or request.user.has_perm("corptools.alliance_corp_manager")
                    or request.user.has_perm("corptools.own_corp_manager")):
                return []

            if character_id == 0:
                character_id = request.user.profile.main_character.character_id
            response, main = get_main_character(request, character_id)

            if not response:
                return 403, _("Permission Denied")

            characters = get_alts_queryset(main)
            character_list = list(
                characters.values_list("character_id", flat=True))

            ref_types = ["player_donation", "player_trading",
                         "contract_price", "corporation_account_withdrawal"]
            wallet_journal = models.CharacterWalletJournalEntry.objects\
                .filter(character__character__in=characters, ref_type__in=ref_types)\
                .select_related('first_party_name', 'second_party_name', 'character__character')\
                .values('first_party_name__name', 'second_party_name__name')\
                .annotate(total_isk=Sum('amount'))\
                .annotate(interactions=Count('amount'))\
                .annotate(fpcat=F('first_party_name__category'))\
                .annotate(spcat=F('second_party_name__category'))\
                .annotate(fpid=F('first_party_name__eve_id'))\
                .annotate(spid=F('second_party_name__eve_id'))\
                .annotate(fpcrp=F('first_party_name__corporation__name'))\
                .annotate(spcrp=F('second_party_name__corporation__name'))\
                .annotate(fpcid=F('first_party_name__corporation__eve_id'))\
                .annotate(spcid=F('second_party_name__corporation__eve_id'))\
                .annotate(fpali=F('first_party_name__alliance__name'))\
                .annotate(spali=F('second_party_name__alliance__name'))\
                .annotate(fpaid=F('first_party_name__alliance__eve_id'))\
                .annotate(spaid=F('second_party_name__alliance__eve_id'))

            output = []
            for w in wallet_journal:
                own_account = False
                if (w['spid'] in character_list and w['fpid'] in character_list):
                    own_account = True

                output.append(
                    {
                        "fpn": w['first_party_name__name'],
                        "firstParty": {
                            "cat": w['fpcat'],
                            "id": w['fpid'],
                            "cid": w['fpcid'],
                            "cn": w['fpcrp'],
                            "aid": w['fpali'],
                            "an": w['fpaid']
                        },
                        "spn": w['second_party_name__name'],
                        "secondParty": {
                            "cat": w['spcat'],
                            "id": w['spid'],
                            "cid": w['spcid'],
                            "cn": w['spcrp'],
                            "aid": w['spali'],
                            "an": w['spaid']
                        },
                        "value": abs(int(w['total_isk'])),
                        "interactions": w['interactions'],
                        "own_account": own_account
                    })

            return output

        @api.get(
            "account/{character_id}/contracts",
            response={200: List[schema.CharacterContract], 403: str},
            tags=self.tags
        )
        def get_character_contracts(request, character_id: int):
            if character_id == 0:
                character_id = request.user.profile.main_character.character_id
            response, main = get_main_character(request, character_id)

            if not response:
                return 403, _("Permission Denied")

            characters = get_alts_queryset(main)

            contracts = models.Contract.objects\
                .filter(character__character__in=characters)\
                .select_related('character__character', 'acceptor_name', 'assignee_name', 'issuer_corporation_name', 'issuer_name') \
                .prefetch_related("contractitem_set", "contractitem_set__type_name").order_by("-date_issued")

            output = []

            for c in contracts:
                _i = []
                for i in c.contractitem_set.all():
                    _i.append({
                        "is_included": i.is_included,
                        "is_singleton": i.is_singleton,
                        "quantity": i.quantity,
                        "raw_quantity": i.raw_quantity,
                        "record_id": i.record_id,
                        "type_name": i.type_name.name,
                    })
                _c = {
                    "character": c.character.character.character_name,
                    "acceptor": c.acceptor_name.name if c.acceptor_id else None,
                    "assignee": c.assignee_name.name if c.assignee_id else None,
                    "contract": c.contract_id,
                    "issuer": c.issuer_name.name,
                    "issuer_corporation_id": c.issuer_corporation_name.name,
                    "days_to_complete": c.days_to_complete,
                    "collateral": c.collateral or 0,
                    "buyout": c.buyout or 0,
                    "price": c.price or 0,
                    "reward": c.reward or 0,
                    "volume": c.volume or 0,
                    "start_location_id": c.start_location_id,
                    "end_location_id": c.end_location_id,
                    "for_corporation": c.for_corporation,
                    "date_accepted": c.date_accepted,
                    "date_completed": c.date_completed,
                    "date_expired": c.date_expired,
                    "date_issued": c.date_issued,
                    "status": c.status,
                    "contract_type": c.contract_type,
                    "availability": c.availability,
                    "title": c.title,
                    "items": _i
                }
                if c.start_location_name:
                    _c.update(
                        {
                            "start_location": {
                                "id": c.start_location_name.location_id,
                                "name": c.start_location_name.location_name
                            }
                        }
                    )
                if c.end_location_name:
                    _c.update(
                        {
                            "end_location": {
                                "id": c.end_location_name.location_id,
                                "name": c.end_location_name.location_name
                            }
                        }
                    )

                output.append(_c)

            return output

        @api.get(
            "account/{character_id}/loyalty",
            response={200: List[schema.LoyaltyPoints], 403: str},
            tags=self.tags
        )
        def get_character_loyalty_Points(request, character_id: int):
            if character_id == 0:
                character_id = request.user.profile.main_character.character_id
            response, main = get_main_character(request, character_id)

            if not response:
                return 403, _("Permission Denied")

            characters = get_alts_queryset(main)

            lp = models.LoyaltyPoint.objects.filter(character__character__in=characters)\
                .select_related('character__character', "corporation")

            output = []

            for _l in lp:
                output.append({
                    "character": _l.character.character,
                    "corporation": {"id": _l.corporation.eve_id, "name": _l.corporation.name},
                    "amount": _l.amount
                })

            return output



from django import forms
from django.test import TestCase

from corptools.templatetags import colors, helpers


class test_form(forms.Form):
    dummy_field = forms.IntegerField()


class TestHelpers(TestCase):

    def test_add_class(self):
        f = test_form()
        test_class = 'class="this_test_class"'
        self.assertFalse(test_class in str(f["dummy_field"]))
        self.assertTrue(test_class in str(
            helpers.addclass(f["dummy_field"], "this_test_class")))

    def test_subtract(self):
        self.assertTrue(helpers.subtract(10, 5) == 5)
        self.assertTrue(helpers.subtract(5, 10) == -5)

    def test_app_settings(self):
        self.assertIsNotNone(helpers.app_setting().get_character_scopes())

    def test_deslug(self):
        self.assertEqual(helpers.deslug("abcd_efgh"), "abcd efgh")
        self.assertEqual(helpers.deslug(15), 15)
        self.assertEqual(helpers.deslug('abcdefgh'), "abcdefgh")

    def test_skill_circles(self):
        lvl5 = '<div role="text" aria-label="Level 5"><span class="fas fa-circle"></span> <span class="fas fa-circle"></span> <span class="fas fa-circle"></span> <span class="fas fa-circle"></span> <span class="fas fa-circle"></span> </div>'
        lvl4 = '<div role="text" aria-label="Level 4"><span class="fas fa-circle"></span> <span class="fas fa-circle"></span> <span class="fas fa-circle"></span> <span class="fas fa-circle"></span> <span class="far fa-circle"></span> </div>'
        lvl3 = '<div role="text" aria-label="Level 3"><span class="fas fa-circle"></span> <span class="fas fa-circle"></span> <span class="fas fa-circle"></span> <span class="far fa-circle"></span> <span class="far fa-circle"></span> </div>'
        lvl2 = '<div role="text" aria-label="Level 2"><span class="fas fa-circle"></span> <span class="fas fa-circle"></span> <span class="far fa-circle"></span> <span class="far fa-circle"></span> <span class="far fa-circle"></span> </div>'
        lvl1 = '<div role="text" aria-label="Level 1"><span class="fas fa-circle"></span> <span class="far fa-circle"></span> <span class="far fa-circle"></span> <span class="far fa-circle"></span> <span class="far fa-circle"></span> </div>'
        lvl0 = '<div role="text" aria-label="Level 0"><span class="far fa-circle"></span> <span class="far fa-circle"></span> <span class="far fa-circle"></span> <span class="far fa-circle"></span> <span class="far fa-circle"></span> </div>'

        lvl3and5 = '<div role="text" aria-label="Level 3"><span class="fas fa-circle"></span> <span class="fas fa-circle"></span> <span class="fas fa-circle"></span> <span class="fas fa-circle text-danger"></span> <span class="fas fa-circle text-danger"></span> </div>'

        self.assertEqual(helpers.skill_level(5, 5), lvl5)
        self.assertEqual(helpers.skill_level(4, 4), lvl4)
        self.assertEqual(helpers.skill_level(3, 3), lvl3)
        self.assertEqual(helpers.skill_level(2, 2), lvl2)
        self.assertEqual(helpers.skill_level(1, 1), lvl1)
        self.assertEqual(helpers.skill_level(0, 0), lvl0)

        self.assertEqual(helpers.skill_level(3, 5), lvl3and5)


class TestColours(TestCase):

    def test_rand_colour(self):
        self.assertEqual(len(colors.random_bright_colour()), 7)
        self.assertTrue(0 <= int(colors.random_bright_colour()[1:3], 16) < 256)
        self.assertTrue(0 <= int(colors.random_bright_colour()[3:5], 16) < 256)
        self.assertTrue(0 <= int(colors.random_bright_colour()[5:], 16) < 256)



from datetime import timedelta

from django.core.management.base import BaseCommand
from django.utils import timezone

from allianceauth.authentication.models import State

from corptools.app_settings import CT_CHAR_NOTIFICATIONS_MODULE
from corptools.models import CharacterAudit


class Command(BaseCommand):
    help = 'Tidy up'

    def add_arguments(self, parser):
        parser.add_argument(
            "--purge",
            action='store_true',
            help='Remove all detatched characters.'
        )

        parser.add_argument(
            "--purge_guest",
            type=str,
            help='Remove all guest state audits, regardless of if they are active or not.'
        )

        parser.add_argument(
            '--purge_inactive',
            action='store_true',
            help='Remove all audits 14 days or more out of date (Based on Character Notification)'
        )

    def handle(self, *args, **options):
        self.stdout.write("Cleaning up CT Audits, this may take a while.")
        if options["purge_inactive"]:
            _c = CharacterAudit.objects.filter(
                character__character_ownership__isnull=True
            )
            self.stdout.write(f"Purging {_c.count()} Unlinked Characters")
            for _d in _c:
                _d.delete()

        if "purge" in options:
            try:
                _s = State.objects.get(name=options["purge"])
            except State.DoesNotExist:
                self.stdout.write(f"`{options['purge']}` state not found!")
            _c = CharacterAudit.objects.filter(
                character__character_ownership__user__profile__state=_s
            )
            self.stdout.write(
                f"This will purge {_c.count()} Characters in the {options['purge']} State")
            _in = input(
                "Are you sure you want to continue? type `yes` to continue: ")
            if _in.lower() == "yes":
                for _d in _c:
                    _d.delete()
            else:
                self.stdout.write("Skipped delete!")

        if options["purge_inactive"]:
            if CT_CHAR_NOTIFICATIONS_MODULE:
                start_time = timezone.now() - timedelta(days=14)
                _c = CharacterAudit.objects.filter(
                    last_update_notif__lte=start_time
                )
                self.stdout.write(f"Purging {_c.count()} inactive Characters")
                for _d in _c:
                    _d.delete()
            else:
                self.stdout.write(
                    "Notification module not active need to use other")



# Generated by Django 2.2.13 on 2020-06-22 06:43

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0015_auto_20200614_0935'),
    ]

    operations = [
        migrations.RenameField(
            model_name='oretaxrates',
            old_name='r8',
            new_name='common_rate',
        ),
        migrations.RenameField(
            model_name='oretaxrates',
            old_name='r64',
            new_name='excptional_rate',
        ),
        migrations.RenameField(
            model_name='oretaxrates',
            old_name='r0',
            new_name='ore_rate',
        ),
        migrations.RenameField(
            model_name='oretaxrates',
            old_name='r32',
            new_name='rare_rate',
        ),
        migrations.RenameField(
            model_name='oretaxrates',
            old_name='r4',
            new_name='ubiquitous_rate',
        ),
        migrations.RenameField(
            model_name='oretaxrates',
            old_name='r16',
            new_name='uncommon_rate',
        ),
        migrations.AddField(
            model_name='miningtax',
            name='flat_tax_rate',
            field=models.DecimalField(
                blank=True, decimal_places=2, default=None, max_digits=5, null=True),
        ),
        migrations.AddField(
            model_name='structure',
            name='last_online_time',
            field=models.DateTimeField(default=None, null=True),
        ),
        migrations.AlterField(
            model_name='miningtax',
            name='tax_rate',
            field=models.ForeignKey(blank=True, default=None, null=True,
                                    on_delete=django.db.models.deletion.CASCADE, to='corptools.OreTaxRates'),
        ),
        migrations.AlterField(
            model_name='oretaxrates',
            name='tag',
            field=models.CharField(default='Mining Tax', max_length=500),
        ),
    ]



from ninja import NinjaAPI

from ... import models


class FittingsApiEndpoints:
    tags = ["Extras"]

    def __init__(self, api: NinjaAPI):
        @api.get(
            "/extras/fit2skills/{fit_id}",
            response={200: dict, 403: str, 404: str, 500: str},
            tags=["Fittings"]
        )
        def get_fit_skills(request, fit_id: str):
            """
                Turn a Fitting into a skill list json.
            """
            if not request.user.is_superuser:
                return 403, {"message": "Hard no pall!"}
            try:
                from fittings.models import Fitting, FittingItem
            except ImportError:
                return 500, "Fittings module not found!"
            try:
                _fit = Fitting.objects.get(
                    id=int(request.resolver_match.kwargs['fit_id']))
            except Fitting.DoesNotExist:
                return 404, "Fitting not found"
            _items = FittingItem.objects.filter(
                fit=_fit).values_list("type_id", flat=True)

            _skill_ids = [182, 183, 184, 1285, 1289, 1290]
            _level_ids = [277, 278, 279, 1286, 1287, 1288]

            _types = models.EveItemDogmaAttribute.objects.filter(
                eve_type_id__in=_items, attribute_id__in=_skill_ids + _level_ids)
            _types = _types | models.EveItemDogmaAttribute.objects.filter(
                eve_type_id=_fit.ship_type_type_id, attribute_id__in=_skill_ids + _level_ids)

            required = {}
            skills = {}
            skill_ids = set()
            for t in _types:
                if t.eve_type_id not in required:
                    required[t.eve_type_id] = {0: {"skill": 0, "level": 0},
                                               1: {"skill": 0, "level": 0},
                                               2: {"skill": 0, "level": 0},
                                               3: {"skill": 0, "level": 0},
                                               4: {"skill": 0, "level": 0},
                                               5: {"skill": 0, "level": 0}}
                a = t.attribute_id
                v = t.value
                if a in _skill_ids:
                    required[t.eve_type_id][_skill_ids.index(a)]["skill"] = v
                elif a in _level_ids:
                    indx = _level_ids.index(a)
                    if required[t.eve_type_id][indx]["level"] < v:
                        required[t.eve_type_id][indx]["level"] = v

                for t in required.values():
                    for s in t.values():
                        if s["skill"]:
                            if s["skill"] not in skills:
                                skills[s["skill"]] = {"skill": "", "level": 0}
                                skill_ids.add(s["skill"])
                            if s["level"] > skills[s["skill"]]["level"]:
                                skills[s["skill"]]["level"] = s["level"]
                out = {}
                for t in models.EveItemType.objects.filter(type_id__in=list(skill_ids)):
                    out[t.name] = skills[t.type_id]['level']

            return {
                "Fitting": _fit.name,
                "skill_list": out
            }

        @api.get(
            "/extras/dogma/{type_id}",
            response={200: dict, 403: str, 404: str, 500: str},
            tags=["Fittings"]
        )
        def get_dogma(request, type_id: str):
            """
                Load dogma for a type_id.
            """
            if not request.user.is_superuser:
                return 403, {"message": "Hard no pall!"}

            _types = models.EveItemDogmaAttribute.objects.filter(
                eve_type_id=type_id)

            dogma = {}
            for t in _types:
                dogma[t.attribute_id] = t.value
            return dogma



from datetime import datetime
from typing import Dict, List, Optional, Union

from ninja import Schema


class Message(Schema):
    message: str


class CharacterUpdate(Schema):
    data: str
    updated: datetime


class Character(Schema):
    character_name: str
    character_id: int
    corporation_id: int
    corporation_name: str
    alliance_id: Optional[int] = None
    alliance_name: Optional[str] = None


class Corporation(Schema):
    corporation_id: int
    corporation_name: str
    alliance_id: Optional[int] = None
    alliance_name: Optional[str] = None


class MenuLink(Schema):
    name: str
    link: str = None


class MenuCategory(MenuLink):
    links: List[MenuLink] = None


class CorpHistory(Schema):
    start: datetime
    corporation: Corporation


class CharacterHistory(Schema):
    character: Character
    history: List[CorpHistory] = None


class EveName(Schema):
    id: int
    name: str
    cat: Optional[str] = None


class CharacterStatus(Schema):
    character: Character
    isk: Optional[float] = None
    sp: Optional[int] = None
    location: Optional[str] = None
    ship: Optional[str] = None
    ship_name: Optional[str] = None
    active: bool
    last_updates: Optional[Dict] = None


class AccountStatus(Schema):
    characters: List[CharacterStatus] = None
    main: Character
    orphan: bool = False


class ValueLabel(Schema):
    value: Union[str, int, float, bool]
    label: str


class CharacterAssetGroups(Schema):
    name: str
    items: List[ValueLabel] = None


class CharacterAssetItem(Schema):
    id: int
    character: Character
    item: EveName
    quantity: int = 0
    location: EveName = None
    expand: bool = None


class CorporationAssetItem(Schema):
    id: int
    item: EveName
    quantity: int = 0
    location: EveName = None
    expand: bool = None


class CharacterClone(Schema):
    name: Optional[str] = None
    location: Optional[EveName] = None
    implants: List[EveName] = None


class CharacterClones(Schema):
    character: Character
    clones: List[CharacterClone] = None
    home: Optional[EveName] = None
    last_station_change: Optional[datetime] = None
    last_clone_jump: Optional[datetime] = None


class Skill(Schema):
    group: str
    skill: str
    sp: int
    level: int
    active: int


class CharacterQueueItem(Schema):
    skill: str
    group: str
    end_level: int
    position: int
    current_level: Optional[int] = 0
    current_sp: Optional[int] = 0
    start_sp: int
    end_sp: int
    start: Optional[datetime] = None
    end: Optional[datetime] = None


class CharacterSkills(Schema):
    character: Character
    skills: Optional[List[Skill]] = None
    total_sp: int
    unallocated_sp: int


class SkillHistory(Schema):
    date: datetime
    total_sp: int
    unallocated_sp: int
    sp: int


class CharacterSkillHistory(Schema):
    character: Character
    history: Optional[List[SkillHistory]] = None


class CharacterQueue(Schema):
    character: Character
    queue: Optional[List[CharacterQueueItem]] = None


class CharacterDoctrines(Schema):
    character: Character
    doctrines: dict
    skills: dict


class CharacterNotification(Schema):
    character: Character
    notification_text: str
    notification_type: str
    timestamp: datetime
    is_read: Optional[bool] = None


class CharacterRoles(Schema):
    character: Character
    director: bool
    station_manager: bool
    personnel_manager: bool
    accountant: bool
    titles: Optional[List[EveName]] = None


class CharacterWalletEvent(Schema):
    character: Character
    id: int
    date: datetime
    first_party: EveName
    second_party: EveName
    ref_type: str
    balance: float
    amount: float
    own_account: Optional[bool] = True
    reason: Optional[str] = None


class LoyaltyPoints(Schema):
    character: Character
    corporation: EveName
    amount: int


class CorporationWalletEvent(Schema):
    division: str
    id: int
    date: datetime
    first_party: EveName
    second_party: EveName
    ref_type: str
    balance: float
    amount: float
    reason: Optional[str] = None


class ContractItems(Schema):
    is_included: bool
    is_singleton: bool
    quantity: Optional[int] = None
    raw_quantity: Optional[int] = None
    record_id: int
    type_name: str


class CharacterContract(Schema):
    character: Union[str, Character]
    id: Optional[int] = None
    contract: Optional[int] = None
    items: Optional[List[ContractItems]] = []
    contract_type: str
    availbility: Optional[str] = None
    tittle: Optional[str] = None

    acceptor: Optional[str] = None
    assignee: Optional[str] = None
    issuer: Optional[str] = None

    issuer_corporation_id: Optional[str] = None

    days_to_complete: int
    collateral: float
    buyout: float
    price: float
    reward: float
    volume: float
    status: str

    start_location_id: Optional[int] = None
    end_location_id: Optional[int] = None

    start_location: Optional[EveName] = None
    end_location: Optional[EveName] = None

    for_corporation: bool

    date_accepted: Optional[datetime] = None
    date_completed: Optional[datetime] = None
    date_expired: Optional[datetime] = None
    date_issued: Optional[datetime] = None


class CharacterOrder(Schema):
    character: Character
    date: datetime
    duration: int
    volume_min: Optional[int] = None
    volume_remain: int
    volume_total: int
    item: EveName
    price: float
    escrow: Optional[float] = None
    buy_order: Optional[bool] = None
    location: Optional[EveName] = None


class CharacterMarket(Schema):
    expired: List[CharacterOrder] = None
    active: List[CharacterOrder] = None
    total_active: float
    total_expired: float


class Contact(Schema):
    character: Character
    contact: EveName
    standing: float
    labels: List[ValueLabel] = None
    blocked: bool
    watched: bool


class StructureService(Schema):
    name: str
    state: str


class Structure(Schema):
    id: int
    owner: Corporation
    name: str
    type: EveName
    services: Optional[List[StructureService]] = None
    location: EveName
    constellation: Optional[EveName] = None
    region: Optional[EveName] = None
    celestial: Optional[str] = None
    fuel_expiry: Optional[datetime] = None
    state: str = None
    state_expiry: Optional[datetime] = None


class OreValue(Schema):
    type: EveName
    quantity: int
    value: float


class Metenox(Schema):
    structure: Structure
    contents: List[OreValue]
    total: int


class FittingItem(Schema):
    type: EveName
    location: str


class CorpStatus(Schema):
    corporation: Corporation
    characters: int
    active: bool
    last_updates: Optional[Dict] = None


class PingStats(Schema):
    members: int
    structures: List[str] = None



# Generated by Django 4.0.10 on 2023-06-30 10:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0088_alter_characterindustryjob_end_date'),
    ]

    operations = [
        migrations.CreateModel(
            name='HighestSPFilter',
            fields=[
                ('id', models.AutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=500)),
                ('description', models.CharField(max_length=500)),
                ('sp_cutoff', models.BigIntegerField(default=5000000000)),
                ('swap_logic', models.BooleanField(default=False)),
            ],
            options={
                'verbose_name': 'Smart Filter: Highest SP Character',
                'verbose_name_plural': 'Smart Filter: Highest SP Character',
            },
        ),
    ]



# Generated by Django 4.0.2 on 2022-05-01 10:08

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0072_alter_corporationaudit_options'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='miningtaxpaymentcorp',
            name='corp',
        ),
        migrations.DeleteModel(
            name='MiningTax',
        ),
        migrations.DeleteModel(
            name='MiningTaxPaymentCorp',
        ),
    ]



from django.urls import path

import allianceauth.urls

from . import views

urlpatterns = allianceauth.urls.urlpatterns

urlpatterns += [
    # Navhelper test urls
    path('main-page/', views.page, name='p1'),
    path('main-page/sub-section/', views.page, name='p1-s1'),
    path('second-page/', views.page, name='p1'),
]



# Generated by Django 3.2.12 on 2022-02-14 09:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0015_factions'),
        ('corptools', '0068_rename_note_text_notification_notification_text'),
    ]

    operations = [
        migrations.CreateModel(
            name='CorptoolsConfiguration',
            fields=[
                ('id', models.AutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('holding_corps', models.ManyToManyField(
                    to='eveonline.EveCorporationInfo')),
            ],
            options={
                'permissions': (('holding_corp_structures', 'Can access configured holding corp structure data.'), ('holding_corp_wallets', 'Can access configured holding corp wallet data.'), ('holding_corp_asset', 'Can access configured holding corp wallet data.')),
                'default_permissions': [],
            },
        ),
    ]



# Generated by Django 4.0.8 on 2022-12-22 02:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0080_alter_characterlocation_character'),
    ]

    operations = [
        migrations.AddField(
            model_name='characterlocation',
            name='current_ship_name',
            field=models.TextField(
                blank=True, default=None, max_length=150, null=True),
        ),
        migrations.AddField(
            model_name='characterlocation',
            name='current_ship_unique',
            field=models.BigIntegerField(blank=True, default=None, null=True),
        ),
    ]



# Generated by Django 4.0.10 on 2023-05-22 09:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0086_characterindustryjob'),
    ]

    operations = [
        migrations.AddField(
            model_name='characteraudit',
            name='last_update_indy',
            field=models.DateTimeField(blank=True, default=None, null=True),
        ),
    ]



# Generated by Django 4.2.16 on 2025-03-11 09:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0117_starbase_name'),
    ]

    operations = [
        migrations.AlterField(
            model_name='skill',
            name='id',
            field=models.BigAutoField(primary_key=True, serialize=False),
        ),
    ]
